package cz.fi.muni.pa165.entity;

import cz.fi.muni.pa165.enums.CountryEnum;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashSet;
import java.util.List;

import static org.testng.Assert.assertEquals;

/**
 * @author Marek Kadlečík
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TeamEntityTest {

    private EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");

    private TeamEntity teamEntity;
    private TournamentEntity plannedTournament;
    private TournamentEntity completedTournament;
    private TeamManagerEntity teamManager;
    private PlayerEntity playerA;

    @BeforeAll
    void init() {
        teamManager = new TeamManagerEntity("Foo");
        teamEntity = new TeamEntity("Autisti III", teamManager);
        plannedTournament = new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        completedTournament = new TournamentEntity("Osaka", CountryEnum.SLOVAKIA, 1000, TournamentFormat.DEFAULT, 10);
        playerA = new PlayerEntity("Bar");

        teamManager.setTeam(teamEntity);
        teamEntity.setPlannedTournaments(new HashSet<>(List.of(plannedTournament)));
        teamEntity.setCompletedTournaments(new HashSet<>(List.of(completedTournament)));
        plannedTournament.addTeamToTournament(teamEntity);
        completedTournament.addTeamToTournament(teamEntity);
        playerA.setTeam(teamEntity);
        teamEntity.addPlayerToTheTeam(playerA);

        EntityManager em = emf.createEntityManager();
        try {
            em.getTransaction().begin();
            em.persist(playerA);
            em.persist(teamManager);
            em.persist(teamEntity);
            em.persist(completedTournament);
            em.persist(plannedTournament);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipTeam() {
        TeamEntity team;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            team = em.find(TeamEntity.class, teamEntity.getId());
            assertEquals(team.getManager(), teamManager);
            assertEquals(team.getPlannedTournaments(), new HashSet<>(List.of(plannedTournament, completedTournament)));
            assertEquals(team.getCompletedTournaments(), new HashSet<>(List.of(completedTournament, plannedTournament)));
            assertEquals(team.getPlayerList(), new HashSet<>(List.of(playerA)));
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }
}
